/////////////////////////////////////////////////////////////////////////////////////
// File:	dhcpconf.h
// Author:	Walt Jones
// Email:	wjones@blarg.net
// Date:	9 July, 2000
// Project:	ResNet Autoreg v2.0
// Purpose:	Interface for the ISC DHCP DAEMON dhcpd.conf configuration file.  Provides
//			static host addition and deletion.  Provides subnet and wounded IP
//			configuration.
/////////////////////////////////////////////////////////////////////////////////////
#include <iostream>
#include <fstream>
#include <stdlib.h>
#include <string>
#include <vector>
#include <time.h>
#include "filelock.h"

// Return types definitions
typedef enum {
	dhcpConfSuccess=1,
	dhcpConfFail=0,
	dhcpConfNoEntry=-1
} dhcpConfResultType;

class dhcpConfInterface
{
private:
	// Variables
	ifstream fin;			// Input filestream
	ofstream fout;			// Output filestream
	char *filename;			// Filename
	char *delims;			// Token delimeters
	fileLock lock;			// File lock

	// Static class variables
	static const int maxStringSize;

	// Methods
	dhcpConfResultType writeVect(vector<string> &vect);

public:
	// Methods
	dhcpConfInterface(const string &file);
	~dhcpConfInterface();
	dhcpConfResultType addEntry(const string &ID, const string &IPaddr, const string &MACaddr);
	dhcpConfResultType removeEntry(const string &ID);
	dhcpConfResultType addDenial(const string &ID, const string &MACaddr);
	dhcpConfResultType removeDenial(const string &ID) {return (removeEntry(ID));};
};

// Initialize static class variables
const int dhcpConfInterface::maxStringSize = 256;

/////////////////////////////////////////////////////////////////////////////////////
// Method:	dhcpConfInterface
// Purpose:	Constructor
// Input:	file - The filename (with full path) to the configuration file to use
// Output:	None
// Return:	None
/////////////////////////////////////////////////////////////////////////////////////
dhcpConfInterface::dhcpConfInterface(const string &file)
{
	// Set up arrays
	filename = new char[maxStringSize + 1];
	delims = new char[5];

	// Set up the delimeter and the filename
	delims[0] = ' ';
	delims[1] = '\t';
	delims[2] = '"';
	delims[3] = ';';
	delims[4] = '\0';
	strcpy(filename, file.c_str());

	// Set up the fileLock
	lock.setFilename(filename);

	return;
}

/////////////////////////////////////////////////////////////////////////////////////
// Method:	~dhcpConfInterface
// Purpose:	Destructor
// Input:	None
// Output:	None
// Return:	None
/////////////////////////////////////////////////////////////////////////////////////
dhcpConfInterface::~dhcpConfInterface()
{
	// Close files
	if (lock.isLocked())
		lock.release();
	fin.close();
	fout.close();
	
	delete filename;
	delete delims;
	return;
}

/////////////////////////////////////////////////////////////////////////////////////
// Method:	addEntry
// Purpose:	Appends a static host to the configuration file
// Input:	ID - Host ID
//			IPaddr - IP address of the host
//			MACaddr - MAC address of the host
// Output:	None
// Return:	dhcpConfSuccess  - The entry was appended successfully
//			dhcpConfFail - An error occured when attempting to open the configuration file
/////////////////////////////////////////////////////////////////////////////////////
dhcpConfResultType dhcpConfInterface::addEntry(const string &ID, const string &IPaddr, const string &MACaddr)
{
	// Open the file
	lock.lock();
	fout.open(filename, ios::app);
	if (fout.fail())
	{
		lock.release();
		return dhcpConfFail;
	}

	// Append the entry
	fout << "host " << ID << " {" << endl;
	fout << "  hardware ethernet " << MACaddr << ";" << endl;
	fout << "  fixed-address " << IPaddr << ";" << endl;
	fout << "}" << endl;

	// Close the file
	fout.close();
	lock.release();
	return dhcpConfSuccess;
}

/////////////////////////////////////////////////////////////////////////////////////
// Method:	removeEntry
// Purpose:	Removes a static host from the configuration file
// Input:	ID - host ID to remove
// Output:	None
// Return:	dhcpConfSuccess - The entry was removed successfully
//			dhcpConfFail - An error occured
//			dhcpConfNoEntry - An entry for the specified ID was not found
/////////////////////////////////////////////////////////////////////////////////////
dhcpConfResultType dhcpConfInterface::removeEntry(const string &ID)
{
	char templine[maxStringSize + 1];		// Temporary line
	char templineCopy[maxStringSize + 1];		// Temporary line
	char *token;					// Token
	vector<string> contents;		// File contents
	vector<string>::iterator end;	// Vector iterators
	bool found = false;				// If the entry has been found

	// Open the file
	lock.lock();
	fin.open(filename);
	if (fin.fail())
	{
		lock.release();
		return dhcpConfFail;
	}

	// Until we reach the end of the file, keep looking for the specified ID. If the
	// line we look at doesn't belong to the specified ID, append it to the vector
	while (!fin.eof())
	{
		fin.getline(templine, maxStringSize, '\n');
		// If the line is length 7 or less, we don't want it
		if (strlen(templine) > 7)
		{
			// Save the templine before we tokenize it
			strcpy(templineCopy,templine);

			// Tokenize the line
			token = strtok(templine, delims);
			
			// Look for the specified ID
			if (!strcmp(token, "host"))
			{
				token = strtok(NULL, delims);
				
				if (token == NULL)
				{
					fin.close();
					lock.release();
					return dhcpConfFail;
				}

				if (!strcmp(token, ID.c_str()))
				{
					// We've found the right ID, so eat space until we hit a '}'
					fin.getline(templine, maxStringSize, '\n');
					while(strcmp(templine, "}"))
						fin.getline(templine, maxStringSize, '\n');
					found = true;
				}
				else
					contents.push_back(string(templineCopy));
			}
			else
				contents.push_back(string(templineCopy));
		}
		else
			contents.push_back(string(templine));
	}

	fin.close();
	lock.release();

	// If we didn't find the entry, it didn't exist
	if (!found)
		return dhcpConfNoEntry;

	// If the last element in the vector is whitespace, dump it
	end = contents.end();
	end--;
	strcpy(templine,string(*end).c_str());
	token = strtok(templine, delims);
	if (token == NULL)
		contents.pop_back();

	// Write the vector to the file
	return writeVect(contents);
}

/////////////////////////////////////////////////////////////////////////////////////
// Method:	addDenial
// Purpose:	Appends a MAC address denial to the configuration file
// Input:	ID - Host ID
//			MACaddr - MAC address of the host
// Output:	None
// Return:	dhcpConfSuccess - The entry was appended successfully
//			dhcpConfFail - An error occured when attempting to open the configuration file
/////////////////////////////////////////////////////////////////////////////////////
dhcpConfResultType dhcpConfInterface::addDenial(const string &ID, const string &MACaddr)
{
	// Open the file
	lock.lock();
	fout.open(filename, ios::app);
	if (fout.fail())
	{
		lock.release();
		return dhcpConfFail;
	}

	// Append the entry
	fout << "host " << ID << " {" << endl;
	fout << "  hardware ethernet " << MACaddr << ";" << endl;
	fout << "  deny booting;" << endl;
	fout << "}" << endl;

	// Close the file
	fout.close();
	lock.release();

	return dhcpConfSuccess;
}

/////////////////////////////////////////////////////////////////////////////////////
// Method:	writeVect
// Purpose:	Writes the contents of a vector to the DHCP config file
// Input:	vect - The vector to write to disk
// Output:	None
// Return:	dhcpConfSuccess - The vector was written successfully to the file
//			dhcpConfFail - An error occured when attempting to open the file specified
/////////////////////////////////////////////////////////////////////////////////////
dhcpConfResultType dhcpConfInterface::writeVect(vector<string> &vect)
{
	vector<string>::iterator end, it;	// Vector iterators

	// Open the file for output
	lock.lock();
	fout.open(filename);
	if (fout.fail())
	{
		lock.release();
		return dhcpConfFail;
	}

	// Go through and write the vector to the file
	end = vect.end();
	for (it=vect.begin(); it != end; it++)
		fout << *it << endl;

	// Close the file
	fout.close();
	lock.release();

	return dhcpConfSuccess;
}
