/////////////////////////////////////////////////////////////////////////////////////
// File:	dhcplease.h
// Author:	Walt Jones
// Email:	wjones@blarg.net
// Date:	2 July, 2000
// Project:	ResNet Autoreg v2.0
// Purpose:	Interface for the ISC DHCP DAEMON dhcpd.leases lease file.  Provides
//			searching for a given IP's lease and for abandoned IPs.
/////////////////////////////////////////////////////////////////////////////////////
#include <iostream>
#include <fstream>
#include <stdlib.h>
#include <string>
#include <vector>
#include <time.h>
#include "filelock.h"

// Return types definitions
typedef enum {
	dhcpLeaseSuccess=1,
	dhcpLeaseFail=0,
	dhcpLeaseNoEntry=-1,
	dhcpLeaseBadMAC=-2,
	dhcpLeaseNoMAC=-3
} dhcpLeaseResultType;

class dhcpLeaseInterface
{
private:
	// Variables
	ifstream fin;		// Input filestream
	char *filename;		// Filename
	char *delims;		// Token delimeters
	fileLock lock;		// File lock

	// Static class variables
	static const int maxStringSize;

public:
	// Methods
	dhcpLeaseInterface(const string &file);
	~dhcpLeaseInterface();
	dhcpLeaseResultType findEntry(const string &IPaddr, string &MACaddr, string &hostname);
	dhcpLeaseResultType findAbandoned(vector<string> &IPaddrs);
};

// Initialize static class variables
const int dhcpLeaseInterface::maxStringSize = 256;

/////////////////////////////////////////////////////////////////////////////////////
// Method:	dhcpLeaseInterface
// Purpose:	Constructor
// Input:	file - The filename (with full path) to the lease file to use
// Output:	None
// Return:	None
/////////////////////////////////////////////////////////////////////////////////////
dhcpLeaseInterface::dhcpLeaseInterface(const string &file)
{
	// Set up arrays
	filename = new char[maxStringSize + 1];
	delims = new char[5];

	// Set up the delimeter and the filename
	delims[0] = ' ';
	delims[1] = '\t';
	delims[2] = '"';
	delims[3] = ';';
	delims[4] = '\0';
	strcpy(filename, file.c_str());

	// Set up the fileLock
	lock.setFilename(filename);

	return;
}

/////////////////////////////////////////////////////////////////////////////////////
// Method:	~dhcpLeaseInterface
// Purpose:	Destructor
// Input:	None
// Output:	None
// Return:	None
/////////////////////////////////////////////////////////////////////////////////////
dhcpLeaseInterface::~dhcpLeaseInterface()
{
	// Close file
	if (lock.isLocked())
		lock.release();
	fin.close();
	
	delete filename;
	delete delims;
	return;
}

/////////////////////////////////////////////////////////////////////////////////////
// Method:	findEntry
// Purpose:	Finds a lease entry for a given IP and returns the MAC address and
//			hostname (if one exists)
// Input:	IPaddr - IP address to search for
// Output:	MACaddr - MAC address of the machine currently assigned the specified IP
//			hostname - hostname of the machine currently assigned the specified IP (left blank if none was listed)
// Return:	dhcpLeaseSuccess - A lease was successfully found for the specified IP
//			dhcpLeaseFail - An error occured
//			dhcpLeaseNoEntry - A lease for the specified IP was not found
//			dhcpLeaseBadMAC - The MAC address for the machine currently assigned the specified IP was not formatted correctly
/////////////////////////////////////////////////////////////////////////////////////
dhcpLeaseResultType dhcpLeaseInterface::findEntry(const string &IPaddr, string &MACaddr, string &hostname)
{
	char templine[maxStringSize + 1];		// Temporary line
	char *token;			// Token

	// Open the file
	lock.lock();
	fin.open(filename);
	if(fin.fail())
	{
		lock.release();
		return dhcpLeaseFail;
	}

	// Until we reach the end of the file, keep looking for the specified lease
	while (!fin.eof())
	{
		fin.getline(templine, maxStringSize, '\n');

		// If the line is length 14 or less, we don't want it
		if (strlen(templine) > 14)
		{
			// Tokenize the line
			token = strtok(templine, delims);
			
			// Look for the specified IP
			if (!strcmp(token, "lease"))
			{
				token = strtok(NULL, delims);
				if (token == NULL)
				{
					fin.close();
					lock.release();
					return dhcpLeaseFail;
				}

				if (!strcmp(token, IPaddr.c_str()))
				{
					// We've found the right IP, so start looking for the MAC address
					fin.getline(templine, maxStringSize, '\n');

					token = strtok(templine, delims);

					while (strcmp(token, "hardware"))
					{
						fin.getline(templine, maxStringSize, '\n');
						token = strtok(templine, delims);
						if (fin.eof())
						{
							fin.close();
							lock.release();
							return dhcpLeaseNoMAC;
						}
					}
					token = strtok(NULL, delims);
					token = strtok(NULL, delims);
					if (token == NULL)
					{
						fin.close();
						lock.release();
						return dhcpLeaseFail;
					}

					// If the MAC address is too long, return false
					if(strlen(token) > 17)
					{
						fin.close();
						lock.release();
						return dhcpLeaseBadMAC;
					}

					// Save the MAC address
					MACaddr = string(token);

					// See if there's a hostname listed
					fin.getline(templine, maxStringSize, '\n');
					token = strtok(templine, delims);
					while (strcmp(token, "client-hostname") && strcmp(token, "}"))
					{
						fin.getline(templine, maxStringSize, '\n');
						token = strtok(templine, delims);
					}

					// If we didn't reach the end of the entry, we found the hostname
					if(strcmp(token, "}"))
					{
						token = strtok(NULL, delims);
						if (token == NULL)
						{
							fin.close();
							lock.release();
							return dhcpLeaseFail;
						}
						hostname = string(token);
					}

					// Close the file and return true
					fin.close();
					lock.release();
					return dhcpLeaseSuccess;
				}
				else
				{
					// Eat everything until we hit a '}'
					fin.getline(templine, maxStringSize, '\n');
					while(strcmp(templine, "}"))
						fin.getline(templine, maxStringSize, '\n');
				}
			}
		}
	}

	// If we made it to here, we didn't find what we wanted
	fin.close();
	lock.release();
	return dhcpLeaseNoEntry;
}

/////////////////////////////////////////////////////////////////////////////////////
// Method:	findAbandoned
// Purpose:	Finds all IPs listed as abandoned
// Input:	None
// Output:	IPaddrs - Vector containing the list of abandoned IPs
// Return:	dhcpLeaseSuccess - The file was successfully searched for abandoned IPs
//			dhcpLeaseFail - An error occured
////////////////////////////////////////////////////////////////////////////////////
dhcpLeaseResultType dhcpLeaseInterface::findAbandoned(vector<string> &IPaddrs)
{
	char templine[maxStringSize + 1];		// Temporary line
	char IPaddr[maxStringSize + 1];			// Temporary IP address
	char *token;			// Token

	// Open the file
	lock.lock();
	fin.open(filename);
	if(fin.fail())
	{
		lock.release();
		return dhcpLeaseFail;
	}

	// Until we reach the end of the file, keep looking for the lease identifier
	while (!fin.eof())
	{
		fin.getline(templine, maxStringSize, '\n');

		// If the line is length 14 or less, we don't want it
		if (strlen(templine) > 14)
		{
			// Tokenize the line
			token = strtok(templine, delims);
			if (token == NULL)
			{
				fin.close();
				lock.release();
				return dhcpLeaseFail;
			}

			// Save the IP address and see if it's abandoned
			if (!strcmp(token, "lease"))
			{
				strcpy(IPaddr, strtok(NULL, delims));

				// Look for the "abandoned" line
				while (strcmp(token, "abandoned")&&strcmp(token, "}"))
				{
					fin.getline(templine, maxStringSize, '\n');
					token = strtok(templine, delims);
					if (token == NULL)
					{
						fin.close();
						lock.release();
						return dhcpLeaseFail;
					}
				}

				// If we found an "abandoned" line, add the IP address to the vector
				if (!strcmp(token, "abandoned"))
				{
					IPaddrs.push_back(string(IPaddr));

					// Eat everything until we hit a '}'
					fin.getline(templine, maxStringSize, '\n');
					while(!strcmp(templine, "}"))
						fin.getline(templine, maxStringSize, '\n');
				}
			}
		}
	}

	fin.close();
	lock.release();
	return dhcpLeaseSuccess;
}
